--[[
	Bags.lua
		Functions used by Bagnon Bags
--]]

--local msg = function(msg) ChatFrame1:AddMessage(msg or 'nil', 0.5, 0.5, 1) end

--[[ A Bag Object Prototype ]]--

local Bag = CreateFrame('Button')
local Frame_mt = {__index = Bag}
local SIZE = 32
local NORMAL_TEXTURE_SIZE = 64 * (SIZE / 37)
local KEY_WIDTH = 18 * (SIZE / 37)

function Bag:Set(parent, bagID)
	self:SetParent(parent)
	self:SetAlpha(parent:GetAlpha())
	self:SetFrameLevel(1)
	self:SetID(bagID)

	if bagID == 0 or bagID == -1 then
		getglobal(self:GetName() .. 'IconTexture'):SetTexture('Interface\\Buttons\\Button-Backpack-Up')
	elseif bagID == -2 then
		getglobal(self:GetName() .. 'IconTexture'):SetTexture('Interface\\Buttons\\UI-Button-KeyRing')
		getglobal(self:GetName() .. 'IconTexture'):SetTexCoord(0, 0.5625, 0, 0.609375)
		self:GetNormalTexture():SetWidth(KEY_WIDTH)
		self:SetWidth(KEY_WIDTH)
	end
	
	if bagID ~= -2 then
		getglobal(self:GetName() .. 'IconTexture'):SetTexCoord(0, 1, 0, 1)
		self:GetNormalTexture():SetWidth(NORMAL_TEXTURE_SIZE)
		self:SetWidth(SIZE)
	end
	self:Update()
end

function Bag:Release()
	TPool.Release(self, 'BagnonBag')
end

--[[ Update Functions ]]--

function Bag:Update()
	if self:GetID() > 0 then
		local invID = ContainerIDToInventoryID(self:GetID())
		local texture = GetInventoryItemTexture("player", invID)
		if texture then
			self.hasItem = 1	
			self:SetCount(GetInventoryItemCount("player", invID))
			SetItemButtonTexture(self, texture)
		else
			self.hasItem = nil	
			self:SetCount(0)
			SetItemButtonTexture(self, nil)
		end
	end
	self:UpdateLock()

	--update tooltip
	if GameTooltip:IsOwned(self) then
		if self.hasItem then
			self:OnEnter()
		else
			GameTooltip:Hide()
			ResetCursor()
		end
	end

	-- Update repair all button status
	if MerchantRepairAllIcon then
		local repairAllCost, canRepair = GetRepairAllCost()
		if canRepair then
			SetDesaturation(MerchantRepairAllIcon, nil)
			MerchantRepairAllButton:Enable()
		else
			SetDesaturation(MerchantRepairAllIcon, 1)
			MerchantRepairAllButton:Disable()
		end
	end
end

function Bag:UpdateLock()
	local locked = IsInventoryItemLocked(ContainerIDToInventoryID(self:GetID()))
	SetItemButtonDesaturated(self, locked)
end

function Bag:UpdateCursor()
	local invID = ContainerIDToInventoryID(self:GetID())
	if CursorCanGoInSlot(invID) then
		self:LockHighlight()
	else
		self:UnlockHighlight()
	end
end

function Bag:UpdateTexture()
	local parent = self:GetParent():GetParent()
	if self:GetID() <= 0 or not parent then return end

	if Bagnon_IsCachedFrame(parent) then
		local player = self:GetParent():GetParent():GetPlayer()
		
		local _, link, count = BagnonDB.GetBagData(player, self:GetID())
		if link then
			local texture = select(9, GetItemInfo(link))
			SetItemButtonTexture(self, texture)
		else
			SetItemButtonTexture(self, nil)
		end
		if count then
			BagnonBag_SetCount(self, count)
		end
	else
		local texture = GetInventoryItemTexture("player", ContainerIDToInventoryID(self:GetID()))
		if texture then
			SetItemButtonTexture(self, texture)
		else
			SetItemButtonTexture(self, nil)
		end
		self:SetCount(GetInventoryItemCount("player", ContainerIDToInventoryID(self:GetID())))
	end
end

function Bag:SetCount(count)
	if self:GetID() <= 0 then return end

	if not count then 
		count = 0 
	end

	local countText = getglobal(self:GetName() .. "Count")
	if count > 0 then
		if count > 9999 then
			countText:SetFont(NumberFontNormal:GetFont(), 10, "OUTLINE")
		elseif count > 999 then
			countText:SetFont(NumberFontNormal:GetFont(), 11, "OUTLINE")
		else
			countText:SetFont(NumberFontNormal:GetFont(), 12, "OUTLINE")
		end
		countText:SetText(count)
		countText:Show()
	else
		countText:Hide()
	end
end

--[[ OnX Functions ]]--

function Bag:OnClick()
	local parent = self:GetParent():GetParent()
	if Bagnon_IsCachedFrame(parent) then return end
	
	local bagID = self:GetID()
	if CursorHasItem() then
		if bagID == KEYRING_CONTAINER then
			PutKeyInKeyRing()
		elseif bagID == 0 then
			PutItemInBackpack()
		else
			PutItemInBag(ContainerIDToInventoryID(bagID))
		end
	else
		parent:ToggleBag(bagID)
	end
end

function Bag:OnDrag()
	local parent = self:GetParent():GetParent()
	if Bagnon_IsCachedFrame(parent) or self:GetID() <= 0 then return end

	PickupBagFromSlot(ContainerIDToInventoryID(self:GetID()))
	PlaySound("BAGMENUBUTTONPRESS")
end

function Bag:OnShow()
	if self:GetID() > 0 and self:GetParent() then
		self:UpdateTexture()
	end
end

--tooltip functions
function Bag:OnEnter()
	local frame = self:GetParent():GetParent()
	frame:HighlightBag(self:GetID(), true)

	Bagnon_AnchorTooltip(self)

	--mainmenubag specific code
	local bagID = self:GetID()
	local cached = Bagnon_IsCachedFrame(frame)
	if bagID == 0 then
		GameTooltip:SetText(TEXT(BACKPACK_TOOLTIP), 1, 1, 1)
	elseif bagID == -1 then
		GameTooltip:SetText('Bank', 1, 1, 1)
	--keyring specific code...again
	elseif bagID == KEYRING_CONTAINER then
		GameTooltip:SetText(KEYRING, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	--cached bags
	elseif cached then
		local link = select(2, BagnonDB.GetBagData(frame:GetPlayer(), bagID))
		if link then
			GameTooltip:SetHyperlink(link)
		else
			GameTooltip:SetText(TEXT(EQUIP_CONTAINER), 1, 1, 1)
		end
	elseif not GameTooltip:SetInventoryItem("player", ContainerIDToInventoryID(bagID)) then
		GameTooltip:SetText(TEXT(EQUIP_CONTAINER), 1, 1, 1)
	end

	if not cached then
		--add the shift click to hide/show tooltip
		if BagnonSets.showTooltips then
			if frame:HasBag(bagID) then
				GameTooltip:AddLine(BAGNON_BAGS_HIDE)
			else
				GameTooltip:AddLine(BAGNON_BAGS_SHOW)
			end
		end
	end
	GameTooltip:Show()
end

function Bag:OnLeave()
	local frame = self:GetParent():GetParent()
	frame:HighlightBag(self:GetID(), nil)

	GameTooltip:Hide()
end

--[[ Bag Constructor ]]--

local function OnEnter() this:OnEnter() end
local function OnLeave() this:OnLeave() end
local function OnShow() this:OnShow() end
local function OnClick() this:OnClick() end
local function OnDrag() this:OnDrag() end

local function Bag_Create(lastCreated)
	local name = "BagnonBag" .. lastCreated
	local bag = CreateFrame("Button", name)

	setmetatable(bag, Frame_mt)

	bag:SetWidth(SIZE)
	bag:SetHeight(SIZE)

	local icon = bag:CreateTexture(name .. "IconTexture", "BORDER")
	icon:SetAllPoints(bag)

	local count = bag:CreateFontString(name .. "Count", "BORDER")
	count:SetFontObject("NumberFontNormal")
	count:SetJustifyH("RIGHT")
	count:SetPoint("BOTTOMRIGHT", bag, "BOTTOMRIGHT", -2, 2)

	local normalTexture = bag:CreateTexture(name .. "NormalTexture")
	normalTexture:SetTexture("Interface\\Buttons\\UI-Quickslot2")
	normalTexture:SetWidth(NORMAL_TEXTURE_SIZE)
	normalTexture:SetHeight(NORMAL_TEXTURE_SIZE)
	normalTexture:SetPoint("CENTER", bag, "CENTER", 0, -1)
	bag:SetNormalTexture(normalTexture)

	local pushedTexture = bag:CreateTexture()
	pushedTexture:SetTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	pushedTexture:SetAllPoints(bag)
	bag:SetPushedTexture(pushedTexture)

	local highlightTexture = bag:CreateTexture()
	highlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square")
	highlightTexture:SetAllPoints(bag)
	bag:SetHighlightTexture(highlightTexture)

	bag:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	bag:RegisterForDrag("LeftButton")

	bag:SetScript("OnShow", OnShow)
	bag:SetScript("OnEnter", OnEnter)
	bag:SetScript("OnLeave", OnLeave)
	bag:SetScript("OnClick", OnClick)
	bag:SetScript("OnDragStart", OnDrag)
	bag:SetScript("OnReceiveDrag", OnClick)

	return bag
end

function BagnonBag_Get()
	return TPool.Get('BagnonBag', Bag_Create)
end