--[[
	BagFrame.lua
--]]

local visibleList = {}
--local msg = function(msg) ChatFrame1:AddMessage(msg or 'nil', 1, 1, 0) end

local BagFrame = CreateFrame('Frame')
local Frame_mt = {__index = BagFrame}
local HEIGHT_SHOWN = 52
local HEIGHT_HIDDEN = 16

function BagFrame:Release()
	TPool.Release(self, 'BagnonBagFrame')
end

function BagFrame:ShowBags()
	self.shown = true
	self:AddBags()
	self:SetHeight(HEIGHT_SHOWN)

	local purchaseFrame = self:GetParent().purchaseFrame
	if purchaseFrame then
		BagnonPurchaseFrame_UpdateVisibility(purchaseFrame, false, true)
	end
	
	local sets = self:GetParent().sets
	if sets then
		sets.showBags = 1
	end
end

function BagFrame:HideBags()
	self.shown = nil
	self:RemoveAllBags()
	self:SetHeight(HEIGHT_HIDDEN)
	self:SetWidth(getglobal(self:GetName() .. 'Toggle'):GetTextWidth())
	
	local purchaseFrame = self:GetParent().purchaseFrame
	if purchaseFrame then
		BagnonPurchaseFrame_UpdateVisibility(purchaseFrame, true, true)
	end
	
	local sets = self:GetParent().sets
	if sets then
		sets.showBags = nil
	end
end

function BagFrame:UpdateBag(bagID)
	local bag = self.bagFrames[bagID]
	if bag then
		bag:Update()
	end
end

function BagFrame:UpdateBags()
	for _, bag in pairs(self.bagFrames) do
		bag:Update()
	end
end

function BagFrame:UpdateLock()
	for _, bag in pairs(self.bagFrames) do
		bag:UpdateLock()
	end
end

function BagFrame:UpdateCursor()
	for _, bag in pairs(self.bagFrames) do
		bag:UpdateCursor()
	end
end

--adds bag buttons, and properly lays them out
function BagFrame:AddBags()
	local width, prev

	for _, bagID in pairs(self.bags) do
		local bag = self.bagFrames[bagID]
		if not bag then 
			bag = BagnonBag_Get()
			bag:Set(self, bagID)
			self.bagFrames[bagID] = bag
		end
		bag:ClearAllPoints()
		if prev then
			bag:SetPoint('BOTTOMLEFT', prev, 'BOTTOMRIGHT', 2, 0)
		else
			bag:SetPoint('TOPLEFT', self, 'TOPLEFT', 0, 2)
		end
		width = (width or 0) + bag:GetWidth() + 2
		prev = bag
	end
	if width then
		self:SetWidth(width)
	else
		self:SetWidth(getglobal(self:GetName() .. 'Toggle'):GetTextWidth())
	end
	self:UpdateBags()
end

function BagFrame:RemoveAllBags()
	for i, bag in pairs(self.bagFrames) do
		bag:Release()
		self.bagFrames[i] = nil
	end
end

--[[ OnX Functions ]]--

function BagFrame:OnShow()
	visibleList[self:GetID()] = self
	if getglobal(self:GetName() .. 'Toggle'):GetText() == BAGNON_HIDEBAGS then
		self:AddBags()
	end
end

function BagFrame:OnHide()
	visibleList[self:GetID()] = nil

	if getglobal(self:GetName() .. 'Toggle'):GetText() == BAGNON_SHOWBAGS then
		self:RemoveAllBags()
	end
end

--[[ Toggle Button ]]--

local function Toggle_OnClick()
	local parent = this:GetParent()
	if parent.shown then
		this:SetText(BAGNON_SHOWBAGS)
		parent:HideBags()
	else
		this:SetText(BAGNON_HIDEBAGS)
		parent:ShowBags()
	end
	parent:GetParent().sizeChanged = true
	parent:GetParent():Layout()
end

local function AddToggle(parent)
	local toggle = CreateFrame('Button', parent:GetName() .. 'Toggle', parent)

	local text = toggle:CreateFontString()
	text:SetPoint('BOTTOMLEFT', toggle)
	text:SetJustifyH('LEFT')
	text:SetFontObject('GameFontNormal')
	toggle:SetFontString(text)
	toggle:SetHighlightTextColor(1, 1, 1)
	toggle:SetTextColor(1, 0.82, 0)
	
	toggle:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	toggle:SetScript('OnClick', Toggle_OnClick)
	
	toggle:SetPoint('TOPLEFT', parent, 'BOTTOMLEFT', 0, 18)
	toggle:SetPoint('BOTTOMRIGHT', parent, 'BOTTOMRIGHT', 0, 4)

	return toggle
end

--[[ Bag Frame Accessors ]]--

local function OnShow() this:OnShow() end
local function OnHide() this:OnHide() end

local function BagFrame_Create(lastCreated)
	local frame = CreateFrame('Frame', 'BagnonBags' .. lastCreated)
	setmetatable(frame, Frame_mt)
	frame.bagFrames = {}
	frame:SetScript('OnShow', OnShow)
	frame:SetScript('OnHide', OnHide)
	
	AddToggle(frame)

	return frame
end

function BagnonBagFrame_Create(parent, bags, showBags)
	local frame = TPool.Get('BagnonBagFrame', BagFrame_Create)
	frame:SetParent(parent)
	frame:SetAlpha(parent:GetAlpha())
	frame:SetFrameLevel(1)
	frame.bags = bags

	if showBags then
		frame.shown = true
		frame:SetHeight(HEIGHT_SHOWN)
		getglobal(frame:GetName() .. 'Toggle'):SetText(BAGNON_HIDEBAGS)
		frame:AddBags()
	else
		frame.shown = nil
		frame:SetHeight(HEIGHT_HIDDEN)
		getglobal(frame:GetName() .. 'Toggle'):SetText(BAGNON_SHOWBAGS)
		frame:SetWidth(getglobal(frame:GetName() .. 'Toggle'):GetTextWidth())
	end
	
	local purchaseFrame = parent.purchaseFrame
	if purchaseFrame then
		BagnonPurchaseFrame_UpdateVisibility(purchaseFrame, not frame.shown, true)
	end
	return frame
end

--[[ Events ]]--

local function UpdateVisibleBags()
	for _, frame in pairs(visibleList) do
		frame:UpdateBags()
	end
end
BEvent:AddAction('BANKFRAME_OPENED', UpdateVisibleBags)
BEvent:AddAction('BAG_UPDATE', UpdateVisibleBags)
BEvent:AddAction('PLAYERBANKSLOTS_CHANGED', UpdateVisibleBags)
BEvent:AddAction('PLAYERBANKBAGSLOTS_CHANGED', UpdateVisibleBags)

BEvent:AddAction('ITEM_LOCK_CHANGED', function()
	for _, frame in pairs(visibleList) do
		frame:UpdateLock()
	end
end)

BEvent:AddAction('CURSOR_UPDATE', function()
	for _, frame in pairs(visibleList) do
		frame:UpdateCursor()
	end
end)