--[[
	vBagnon\purchaseFrame.lua
		A frame for purchasing bank bag slots
--]]

local usedList = {}

--[[ Local Functions ]]--

local function BuySlot()
	if not StaticPopupDialogs["CONFIRM_BUY_BANK_SLOT_BANKNON"] then
		StaticPopupDialogs["CONFIRM_BUY_BANK_SLOT_BANKNON"] = {
			text = TEXT(CONFIRM_BUY_BANK_SLOT),
			button1 = TEXT(YES),
			button2 = TEXT(NO),

			OnAccept = function() PurchaseSlot() end,

			OnShow = function()
				MoneyFrame_Update(this:GetName().."MoneyFrame", GetBankSlotCost(GetNumBankSlots()))
			end,

			hasMoneyFrame = 1,
			timeout = 0,
			hideOnEscape = 1,
		}
	end
	PlaySound("igMainMenuOption")
	StaticPopup_Show("CONFIRM_BUY_BANK_SLOT_BANKNON")
end

local function PurchaseFrame_Create(lastCreated)
	local name = 'BagnonPurchase' .. lastCreated

	local frame = CreateFrame('Frame', 'BagnonPurchase' .. lastCreated)
	frame:SetWidth(164); frame:SetHeight(22)

	local purchaseButton = CreateFrame('Button', name .. 'Purchase', frame, 'UIPanelButtonTemplate')
	purchaseButton:SetWidth(124); purchaseButton:SetHeight(22)
	purchaseButton:SetPoint('LEFT', frame)
	purchaseButton:SetScript('OnClick', BuySlot)
	purchaseButton:SetText(BANKSLOTPURCHASE)
	
	local costFrame = CreateFrame('Frame', name .. 'Cost', frame, 'SmallMoneyFrameTemplate')
	costFrame:SetPoint('LEFT', purchaseButton, 'RIGHT', 2, 0)
	local oldthis = this
	this = costFrame
	MoneyFrame_SetType("STATIC")
	this = oldthis

	return frame
end

--[[ Update Functions ]]--

function BagnonPurchaseFrame_UpdateVisibility(frame, hide, dontUpdate)
	local parent = frame:GetParent()
	if parent then
		local wasVisible = frame:IsShown()
		local _, full = GetNumBankSlots()
		local bagFrame = parent.bagFrame
		if full or hide or not(Bagnon_PlayerAtBank() and parent:GetPlayer() == UnitName('player')) then 
			frame:Hide()
			if wasVisible and not dontUpdate then
				parent.sizeChanged = true
				parent:Layout()
			end
		elseif not bagFrame or bagFrame.shown then
			frame:Show()
			if not(wasVisible or dontUpdate)then
				parent.sizeChanged = true
				parent:Layout()
			end
		else
			frame:Hide()
			if wasVisible and not dontUpdate then
				parent.sizeChanged = true
				parent:Layout()
			end
		end
	end
end

local function BagnonPurchaseFrame_UpdateSlotCost(frame)
	local costFrameName = frame:GetName() .. 'Cost'
	local costFrame = getglobal(costFrameName)
	local cost = GetBankSlotCost(GetNumBankSlots())
	if GetMoney() >= cost then
		SetMoneyFrameColor(costFrameName, 1, 1, 1)
	else
		SetMoneyFrameColor(costFrameName, 1, 0.1, 0.1)
	end
	MoneyFrame_Update(costFrameName, cost)
	
	BagnonPurchaseFrame_UpdateVisibility(frame)
end

--[[ Constructor/Destructor ]]--

function BagnonPurchaseFrame_Create(parent)
	local frame = TPool.Get('BagnonPurchaseFrame', PurchaseFrame_Create)
	frame:SetParent(parent)
	frame:SetAlpha(parent:GetAlpha())
	frame:SetFrameLevel(1)
	
	BagnonPurchaseFrame_UpdateSlotCost(frame)
	
	usedList[frame.id] = frame
	return frame
end

function BagnonPurchaseFrame_Release(frame)
	TPool.Release(frame, 'BagnonPurchaseFrame')
end

--[[ Events ]]--

local function UpdateSlotCostForAll()
	for _, frame in pairs(usedList) do
		BagnonPurchaseFrame_UpdateSlotCost(frame)
	end
end
BEvent:AddAction('PLAYER_MONEY', UpdateSlotCostForAll)
BEvent:AddAction('PLAYERBANKBAGSLOTS_CHANGED', UpdateSlotCostForAll)

local function UpdateVisibilityForAll()
	for _, frame in pairs(usedList) do
		BagnonPurchaseFrame_UpdateVisibility(frame)
	end
end
BEvent:AddAction('BANKFRAME_OPENED', UpdateSlotCostForAll)
BEvent:AddAction('BANKFRAME_CLOSED', UpdateSlotCostForAll)